@@ -51,7 +51,7 @@ class AddressInUseException(Exception): pass
 
 class StopNode(Exception): pass
 
-VERSION = [0, 0, 2]
+VERSION = [0, 0, 3]
 
 class BaseNode(asyncore.dispatcher, object):
 
@@ -183,7 +183,11 @@ class BaseNode(asyncore.dispatcher, object):
         else:
             source = 'node'
         message = '(%s) %s' % (source, message)
-        print >>self._log, message[:100]
+
+        #if message.find('Traceback') == -1:
+        #    message = message[:100]
+
+        print >>self._log, message
 
     # Relaying
     # @TODO: not implemented yet and won't be made available until
